/*
 * bv2 project
 * Created on Feb 14, 2005
 *
 * Copyright (C) 2005
 * Gergely Varro
 * Department of Computer Science and Information Theory
 * Budapest University of Technology and Economics
 * Budapest, Hungary
 */
package hu.bme.cs.benchmark.mutex.fujaba;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import many.Container;
import many.Process;
/**
 * @author gergo
 *
 * Mutex benchmark for Fujaba
 * 
 * Short transformation sequence (STS)
 * - zero-to-many type metamodel
 * - without parameter passing
 */
public class STSmany {
    private static Logger logger = Logger.getLogger(STSmany.class);

    public static void dump(Container c) {
        java.util.Iterator i = c.iteratorOfProcesses();
        while(i.hasNext()) {
            Process p = (Process) i.next();
            System.out.println(p.toString());
            java.util.Iterator j = p.iteratorOfNext();
            while(j.hasNext()) {
                System.out.println(p.toString() + " -next-> " + j.next().toString());
            }
            j = p.iteratorOfRequests();
            while(j.hasNext()) {
                System.out.println(p.toString() + " -request-> " + j.next().toString());
            }
            j = p.iteratorOfHolds();
            while(j.hasNext()) {
                System.out.println(p.toString() + " <-held_by- " + j.next().toString());
            }
            j = p.iteratorOfReleases();
            while(j.hasNext()) {
                System.out.println(p.toString() + " <-release- " + j.next().toString());
            }
            j = p.iteratorOfTokens();
            while(j.hasNext()) {
                System.out.println(p.toString() + " <-token- " + j.next().toString());
            }
            j = p.iteratorOfBlockedBy();
            while(j.hasNext()) {
                System.out.println(p.toString() + " <-blocked- " + j.next().toString());
            }
            System.out.println("");
        }
    }
    
    public static void main(String[] args) {
        Container c = new Container();
        Process p1,p2;
        int N = 0;
        
        try {
            N = Integer.parseInt(args[0]);
        } catch (Exception ex) {
            System.err.println("Sample usage: java " + STSmany.class.getName() + " 4");
            System.exit(1);
        }
        BasicConfigurator.configure();
       
        /* Initial model generation */
        p1 = new Process();
        c.addToProcesses(p1);
        p2 = new Process();
        c.addToProcesses(p2);
        p1.addToNext(p2);
        p2.addToNext(p1);

        // Run tests
        try {
            int cnt = 0;
            for(cnt = 0; cnt < N-2; cnt++) {
                if(logger.isDebugEnabled()) {
                    logger.debug("newRule: init: " + System.nanoTime());
                    c.newRule();
                    logger.debug("newRule: update: " + System.nanoTime());
                } else {
                    c.newRule();
                }
            }

            for(cnt = 0; cnt < 1; cnt++) {
                if(logger.isDebugEnabled()) {
                    logger.debug("mountRule: init: " + System.nanoTime());
                    c.mountRule();
                    logger.debug("mountRule: update: " + System.nanoTime());
                } else {
                    c.mountRule();
                }
            }
            
            for(cnt = 0; cnt < N; cnt++) {
                if(logger.isDebugEnabled()) {
                    logger.debug("requestRule: init: " + System.nanoTime());
                    c.requestRule();
                    logger.debug("requestRule: update: " + System.nanoTime());
                } else {
                    c.requestRule();
                }
            }
           
            for(cnt = 0; cnt < N; cnt++) {
                if(logger.isDebugEnabled()) {
                    logger.debug("takeRule: init: " + System.nanoTime());
                    c.takeRule();
                    logger.debug("takeRule: update: " + System.nanoTime());
                    logger.debug("releaseRule: init: " + System.nanoTime());
                    c.releaseRule();
                    logger.debug("releaseRule: update: " + System.nanoTime());
                    logger.debug("giveRule: init: " + System.nanoTime());
                    c.giveRule();
                    logger.debug("giveRule: update: " + System.nanoTime());
                } else {
                    c.takeRule();
                    c.releaseRule();
                    c.giveRule();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
