/*
 * bv2 project
 * Created on Feb 14, 2005
 *
 * Copyright (C) 2005
 * Gergely Varro
 * Department of Computer Science and Information Theory
 * Budapest University of Technology and Economics
 * Budapest, Hungary
 */
package hu.bme.cs.benchmark.mutex.fujaba;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import many.Container;
import many.Process;
import many.Resource;
/**
 * @author gergo
 *
 * Mutex benchmark for Fujaba
 * 
 * Short transformation sequence (STS)
 * - zero-to-many type metamodel
 * - with parameter passing
 */
public class STSmanyPP {
    private static Logger logger = Logger.getLogger(STSmanyPP.class);
    
    public static void main(String[] args) {
        Container c = new Container();
        Process p1,p2;
        int N = 0;
        
        try {
            N = Integer.parseInt(args[0]);
        } catch (Exception ex) {
            System.err.println("Sample usage: java " + STSmanyPP.class.getName() + " 4");
            System.exit(1);
        }
        BasicConfigurator.configure();
       
        /* Initial model generation */
        p1 = new Process();
        c.addToProcesses(p1);
        p2 = new Process();
        c.addToProcesses(p2);
        p1.addToNext(p2);
        p2.addToNext(p1);
        
        // Run tests
        try {
            int cnt = 0;
            for(cnt = 0; cnt < N-2; cnt++) {
                if(logger.isDebugEnabled()) {
                    logger.debug("newRule: init: " + System.nanoTime());
                    c.newRule();
                    logger.debug("newRule: update: " + System.nanoTime());
                } else {
                    c.newRule();
                }
            }

            for(cnt = 0; cnt < 1; cnt++) {
                if(logger.isDebugEnabled()) {
                    logger.debug("mountRule: init: " + System.nanoTime());
                    c.mountRule();
                    logger.debug("mountRule: update: " + System.nanoTime());
                } else {
                    c.mountRule();
                }
            }
            
            for(cnt = 0; cnt < N; cnt++) {
                if(logger.isDebugEnabled()) {
                    logger.debug("requestRule: init: " + System.nanoTime());
                    c.requestRule();
                    logger.debug("requestRule: update: " + System.nanoTime());
                } else {
                    c.requestRule();
                }
            }
           
            for(cnt = 0; cnt < N; cnt++) {
                if(logger.isDebugEnabled()) {
                    logger.debug("takeRulePP: init: " + System.nanoTime());
                    Resource r = c.takeRulePP();
                    logger.debug("takeRulePP: update: " + System.nanoTime());
                    logger.debug("releaseRulePP: init: " + System.nanoTime());
                    r.releaseRulePP();
                    logger.debug("releaseRulePP: update: " + System.nanoTime());
                    logger.debug("giveRulePP: init: " + System.nanoTime());
                    r.giveRulePP();
                    logger.debug("giveRulePP: update: " + System.nanoTime());
                } else {
                    Resource r = c.takeRulePP();
                    r.releaseRulePP();
                    r.giveRulePP();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
